"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
}
Object.defineProperty(exports, "__esModule", { value: true });
const url_1 = __importDefault(require("url"));
const crypto_1 = __importDefault(require("crypto"));
const infrastructure_1 = __importDefault(require("./infrastructure"));
const error_1 = require("./utils/error");
class Retrieval extends infrastructure_1.default {
    title() {
        return new Promise((resolve, reject) => {
            this.push(() => __awaiter(this, void 0, void 0, function* () {
                try {
                    resolve(yield this.page.title());
                }
                catch (error) {
                    reject(error);
                }
            }));
        });
    }
    text(selector = 'body') {
        return new Promise((resolve, reject) => {
            this.push(() => __awaiter(this, void 0, void 0, function* () {
                try {
                    const text = yield this.page.evaluate(
                    /* Instrumenting cannot be executed in browser. */
                    /* istanbul ignore next */
                    sel => document.querySelector(sel).textContent, selector);
                    resolve(text);
                }
                catch (error) {
                    reject(error);
                }
            }));
        });
    }
    html(selector = 'html') {
        return new Promise((resolve, reject) => {
            this.push(() => __awaiter(this, void 0, void 0, function* () {
                try {
                    const html = yield this.page.evaluate(
                    /* Instrumenting cannot be executed in browser. */
                    /* istanbul ignore next */
                    sel => document.querySelector(sel).innerHTML, selector);
                    resolve(html);
                }
                catch (error) {
                    reject(error);
                }
            }));
        });
    }
    attribute(selector, attribute) {
        return new Promise((resolve, reject) => {
            this.push(() => __awaiter(this, void 0, void 0, function* () {
                try {
                    const value = yield this.page.evaluate(
                    /* Instrumenting cannot be executed in browser. */
                    /* istanbul ignore next */
                    (sel, attr) => document
                        .querySelector(sel)
                        .getAttribute(attr), selector, attribute);
                    resolve(value);
                }
                catch (error) {
                    reject(error);
                }
            }));
        });
    }
    style(selector, property) {
        return new Promise((resolve, reject) => {
            this.push(() => __awaiter(this, void 0, void 0, function* () {
                try {
                    const value = yield this.page.evaluate(
                    /* Instrumenting cannot be executed in browser. */
                    /* istanbul ignore next */
                    (sel, prop) => document
                        .querySelector(sel)
                        .style.getPropertyValue(prop), selector, property);
                    resolve(value);
                }
                catch (error) {
                    reject(error);
                }
            }));
        });
    }
    value(selector, newValue) {
        if (newValue) {
            this.push(() => __awaiter(this, void 0, void 0, function* () {
                yield this.page.evaluate(
                /* Instrumenting cannot be executed in browser. */
                /* istanbul ignore next */
                (sel, val) => document
                    .querySelector(sel)
                    .value = val, selector, newValue);
            }), error_1.prepareStackTrace());
            return this;
        }
        else {
            return this.attribute(selector, 'value');
        }
    }
    hasClass(selector, className) {
        return new Promise((resolve, reject) => {
            this.push(() => __awaiter(this, void 0, void 0, function* () {
                try {
                    const exists = yield this.page.evaluate(
                    /* Instrumenting cannot be executed in browser. */
                    /* istanbul ignore next */
                    (sel, cls) => document
                        .querySelector(sel)
                        .classList.contains(cls), selector, className);
                    resolve(exists);
                }
                catch (error) {
                    reject(error);
                }
            }));
        });
    }
    url() {
        return new Promise(fulfill => {
            this.push(() => fulfill(this.page.url()));
        });
    }
    queryString(key) {
        return new Promise(fulfill => {
            this.push(() => {
                const { query } = url_1.default.parse(this.page.url(), true);
                fulfill(query[key]);
            });
        });
    }
    cookie() {
        return new Promise((resolve, reject) => {
            this.push(() => __awaiter(this, void 0, void 0, function* () {
                try {
                    resolve((yield this.page.cookies())[0]);
                }
                catch (error) {
                    reject(error);
                }
            }));
        });
    }
    cookies() {
        return new Promise((resolve, reject) => {
            this.push(() => __awaiter(this, void 0, void 0, function* () {
                try {
                    resolve(yield this.page.cookies());
                }
                catch (error) {
                    reject(error);
                }
            }));
        });
    }
    isVisible(selector) {
        return new Promise((resolve, reject) => {
            this.push(() => __awaiter(this, void 0, void 0, function* () {
                try {
                    const result = yield this.page.evaluate(
                    /* Instrumenting cannot be executed in browser. */
                    /* istanbul ignore next */
                    sel => document.querySelector(sel).style.display, selector);
                    resolve(result !== 'none');
                }
                catch (error) {
                    reject(error);
                }
            }));
        });
    }
    isPresent(selector) {
        return new Promise((resolve, reject) => {
            this.push(() => __awaiter(this, void 0, void 0, function* () {
                try {
                    const element = yield this.page.evaluate(
                    /* Instrumenting cannot be executed in browser. */
                    /* istanbul ignore next */
                    sel => document.querySelector(sel), selector);
                    resolve(element !== null);
                }
                catch (error) {
                    reject(error);
                }
            }));
        });
    }
    find(selector, fn, ...args) {
        const random = crypto_1.default.randomBytes(10).toString('hex');
        this.push(() => __awaiter(this, void 0, void 0, function* () {
            yield this.page.evaluate(
            /* Instrumenting cannot be executed in browser. */
            /* istanbul ignore next */
            (sel, id) => document
                .querySelector(sel)
                .setAttribute('data-rize', id), selector, random);
        }));
        return fn.call(this, `[data-rize="${random}"]`, ...args);
    }
    findAll(selector, index, fn, ...args) {
        const random = crypto_1.default.randomBytes(10).toString('hex');
        this.push(() => __awaiter(this, void 0, void 0, function* () {
            yield this.page.evaluate(
            /* Instrumenting cannot be executed in browser. */
            /* istanbul ignore next */
            (sel, i, id) => document
                .querySelectorAll(sel)[i]
                .setAttribute('data-rize', id), selector, index, random);
        }));
        return fn.call(this, `[data-rize="${random}"]`, ...args);
    }
    findByXPath(expression, index, fn, ...args) {
        const random = crypto_1.default.randomBytes(10).toString('hex');
        this.push(() => __awaiter(this, void 0, void 0, function* () {
            yield this.page.evaluate(
            /* Instrumenting cannot be executed in browser. */
            /* istanbul ignore next */
            (expr, i, id) => {
                const elements = document.evaluate(expr, document, null, XPathResult.ANY_TYPE, null);
                let element = elements.iterateNext();
                let it = 0;
                while (element && it < i) {
                    element = elements.iterateNext();
                    it++;
                }
                void element.setAttribute('data-rize', id);
            }, expression, index, random);
        }));
        return fn.call(this, `[data-rize="${random}"]`, ...args);
    }
    findWithText(selector, text, index, fn, ...args) {
        const random = crypto_1.default.randomBytes(10).toString('hex');
        this.push(() => __awaiter(this, void 0, void 0, function* () {
            yield this.page.evaluate(
            /* Instrumenting cannot be executed in browser. */
            /* istanbul ignore next */
            (sel, t, i, id) => {
                const element = Array
                    .from(document.querySelectorAll(sel))
                    .filter(el => (el.textContent || '').includes(t))[i];
                element.setAttribute('data-rize', id);
            }, selector, text, index, random);
        }));
        return fn.call(this, `[data-rize="${random}"]`, ...args);
    }
    viewport() {
        return new Promise(fulfill => this.push(() => fulfill(this.page.viewport())));
    }
}
exports.default = Retrieval;
